#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include <string.h>

#include "Aria.h"
#include "ArNetworking.h"

#include <opencv/cv.h>
#include <opencv/highgui.h>

/** @example calVideoExample.cpp
 *
 *  This simple example requests video images from the server
 *  and saves them repeatedly to a file named "camera.jpg".
 *  Connect to SAVserver, ACTS, or a custom video server.
 *  It uses OpenCv to open a window and display the latest camera image
 *  It allows the user to pan tilt and zoom the camera from the keyboard.
 *  It also allows the user to save an image and the ptz settings for the image.
 */
using namespace cv;

ArClientBase client;

int global_count=0; /* index for frames */
int global_max = 20; /* max number of frames to store */
char froot[100] = "frame"; /* file name root to use */
char fname[100]; 

bool calFlag=false, calTest=false, readSpaceValues=false;

IplImage *img;
CvMat *imagePoints=0,*spacePoints=0,*hImage2Space=0; /* cal info */

CvPoint2D32f iP[100];
CvPoint2D32f sP[100];  

CvMat *uvPoints=0,*xyzPoints=0; /* cal test info */


int N = 0;


unsigned char jpeg[50000];
int jpegSize;

ArRobot robot;                           // declare robot object

//ArDPPTU myPTZ(&robot);
ArVCC4 myPTZ(&robot);



/******************************/

void doReadSpaceValues() {
  double a,b;
  printf("Enter the 3D coordinates (assume Z is zero): X Y <ret>\n");      
  scanf("%lf %lf",&a, &b);
  printf("Got %lf %lf\n",a,b);
  sP[N].x = a;  sP[N].y=b;
  N++;
  readSpaceValues=false;
}

void mouseCallback(int ev, int x, int y, int fl, void *params)
{ double px=0,py=0,pz=0;
  if (ev==CV_EVENT_LBUTTONUP) {
    printf("pixel at (%d,%d)\n",x,y);
    if (calFlag) {
      iP[N].x = x; iP[N].y=y;
      readSpaceValues = true;
    }
    if (calTest && hImage2Space!=0 && uvPoints!=0 && xyzPoints!=0) {
      cvSet1D(uvPoints,0,cvScalar(x,y,1,1));
      cvPerspectiveTransform(uvPoints,xyzPoints,hImage2Space);

      px = cvGet1D(xyzPoints,0).val[0];
      py = cvGet1D(xyzPoints,0).val[1];
	printf("Point is at (%lf,%lf,%lf)\n",px,py,0.0);
      
    } else printf("Calibration not initialized\n");
  }
}


/*
 Write JPEG data to a file
 */
void saveJpeg(char *fname, unsigned char *jdata, int jsize) {
  FILE *file;

  if ((file = fopen(fname, "wb+")) != NULL)
  {
    fwrite(jdata, jsize, 1, file);
    fclose(file);
  }
  else
    printf("Could not write jpeg file %s\n",fname);
}

/*
 Slow version: stores jpeg in file and then loads into open cv
 */

void jpegHandler(ArNetPacket *packet)
{
  unsigned int width;
  unsigned int height;

  width = packet->bufToUByte2();
  height = packet->bufToUByte2();
  //printf("Got jpeg of %d x %d\n", width, height);
  jpegSize = packet->getDataLength() - packet->getDataReadLength();
  //printf("Jpeg %d long\n", jpegSize);
  packet->bufToData((char *)jpeg, jpegSize);
  sprintf(fname,"camera.jpg",froot);
  saveJpeg(fname,jpeg,jpegSize);
  img = cvLoadImage(fname);
  cvShowImage("Camera",img);
}

/* Save camera PTZ settings to file fname */
void savePTZ(char *fname) {
  FILE *out = fopen(fname,"w");
  double p,t,z;
  if (out==NULL) {
    printf("Error opening %s for write\n",fname);
    return;
  }
  p = myPTZ.getPan(), t = myPTZ.getTilt(), z = myPTZ.getZoom();
  fprintf(out,"%lf %lf %lf\n",p,t,z);
  fclose(out);
  return;
}

/* Save calibrated homography settings to file fname */
void saveHomography(char *fname) {
  FILE *out = fopen(fname,"w");
  int i,j;
  if (out==NULL) {
    printf("Error opening %s for write\n",fname);
    return;
  }
  for (i=0; i<3; i++) 
    for (j=0; j<3; j++) 
      fprintf(out,"%lf ",cvGet2D(hImage2Space,i,j).val[0]);
  fprintf(out,"\n");
  fclose(out);
  return;
}

void readHomography(char *fname) {
  FILE *src = fopen(fname,"r");
  int i,j; double x;
  if (src==NULL) {
    printf("Error opening %s for read\n",fname);
    return;
  }

  if (hImage2Space==0) hImage2Space=cvCreateMat(3,3,CV_32FC1);
  cvZero(hImage2Space);

  for (i=0; i<3; i++) 
    for (j=0; j<3; j++)  {
      fscanf(src,"%lf ",&x);
      cvSet2D(hImage2Space,i,j, cvScalarAll(x));
   }
  fclose(src);
  return;
}

void saveCurrentImage(char *savedname) {

  sprintf(froot,"%s_%d.jpg",savedname,global_count);
  saveJpeg(froot,jpeg,jpegSize);
  printf("Current image saved to %s\n",froot);

  sprintf(froot,"%sPTZ_%d.txt",savedname,global_count);
  savePTZ(froot);
  printf("Current PTZ settings saved to %s\n",froot);

  if (hImage2Space!=0) {
  sprintf(froot,"%sHMAT_%d.txt",savedname,global_count++);
  saveHomography(froot);
  printf("Current Homography settings saved to %s\n",froot);
  }  else global_count++;
}

/*
 classify key into the arrow keys and +/- keys and a few others
 */

int classifyKeyPress(int key) {
  int ret=-1;
    if (key>0) {
#ifdef WIERDKEYB
      if (key==65363 || key==65430) ret = 1; //printf("left arrow\n");
      if (key==65361 || key==65432) ret = 2; //printf("right arrow\n");
      if (key==65362 || key==65431) ret = 3; //printf("up arrow\n");
      if (key==65364 || key==65433) ret = 4; //printf("left arrow\n");
      if (key==65631 || key==65453) ret = 5; //printf("minus\n");
      if (key==65579 || key==65451) ret = 6; //printf("plus\n");
      if (key==65360 )              ret = 7; //printf("home\n");
      if (key==   10 )              ret = 8; //printf("ret\n");
      if (key==65599 )              ret = 9; //printf("question mark\n");
      if (key==65603 || key==99   )  ret = 10; //printf("C or c\n");
      if (key==65604 || key==100   ) ret = 11; //printf("D or d\n");
      if (key==65605 || key==101   ) ret = 12; //printf("E or e\n");
      if (key==65620 || key==116   ) ret = 13; //printf("t or t\n");
      if (key==65606 || key==102   ) ret = 14; //printf("F or f\n");
#endif
      if (key==1113937 || key==65430) ret = 1; //printf("left arrow\n");
      if (key==1113939 || key==65432) ret = 2; //printf("right arrow\n");
      if (key==1113938 || key==65431) ret = 3; //printf("up arrow\n");
      if (key==1113940 || key==65433) ret = 4; //printf("left arrow\n");
      if (key==65631 || key==65453) ret = 5; //printf("minus\n");
      if (key==65579 || key==65451) ret = 6; //printf("plus\n");
      if (key==65360 )              ret = 7; //printf("home\n");
      if (key==1048536 )              ret = 8; //printf("ret\n");
      if (key==65599 )              ret = 9; //printf("question mark\n");
      if (key==1048675 || key==99   )  ret = 10; //printf("C or c\n");
      if (key==65604 || key==100   ) ret = 11; //printf("D or d\n");
      if (key==1048677 || key==101   ) ret = 12; //printf("E or e\n");
      if (key==1048692 || key==116   ) ret = 13; //printf("t or t\n");
      if (key==65606 || key==102   ) ret = 14; //printf("F or f\n");

    }
    return ret;
}

void printCalStart() {
  printf("Calibration in progress.\n");
  printf("Enter the x and y part of the (assumes z=0) 3D location, then click on the image point \n");
  printf("Type 'e' to end. Type 'd' to delete the last point.\n");
}

void printCalEnd() {
  printf("Calibration data collected.\n");
  printf("The calibration image, PTZ, perspective matrix written out.\n");
  printf("To evaluate calibration, type t\n");
}

void startCalTest() {
  if (hImage2Space==0) {
    printf("Do calibration first\n");
    return;
  }
  printf("Click on the image and the 3D posn will be printed out\n");
  if (uvPoints==0)  
    uvPoints = cvCreateMat(1,1,CV_32FC2);
  if (xyzPoints==0) 
    xyzPoints = cvCreateMat(1,1,CV_32FC2);
}

void printHelp() {
  printf("Select the video window and press one of the following:\n");
  printf(" left arrow pans left \n");
  printf(" right arrow pans right \n");
  printf(" up arrow tilts up \n");
  printf(" down arrow tilts down \n");
  printf(" + zooms in \n");
  printf(" - zooms out \n");
  printf(" home resets the camera (takes a minute or so) \n");
  printf(" return saves an image to savedimage<n>.jpg where n increments \n");
  printf(" ? prints this message \n");
}

void calStart() {
  int i,j;
  for (i=0; i<100; i++) for (j=0; j<4; j++) {
      iP[i].x=iP[i].y=sP[i].x=sP[i].y=0;
    }
  N=0;
}

void calEnd() {
  int i;
  printf("There are %d calibration points\n Summary:\n",N);
  if (N==0) return;
  /*
  if (imagePoints!=0) cvReleaseMat(&imagePoints);
  else imagePoints=cvCreateMat(N,3,CV_32FC1);
  if (spacePoints!=0) cvReleaseMat(&spacePoints);
  else spacePoints=cvCreateMat(N,3,CV_32FC1);
  */
  if (hImage2Space==0) hImage2Space=cvCreateMat(3,3,CV_32FC1);
  cvZero(hImage2Space);
  /*
  for (i=0; i<N; i++) {
    cvSet2D(imagePoints,i,0,cvScalarAll(iP[i].x));
    cvSet2D(imagePoints,i,1,cvScalarAll(iP[i].y));
    cvSet2D(imagePoints,i,2,cvScalarAll(1));
    cvSet2D(spacePoints,i,0,cvScalarAll(sP[i].x));
    cvSet2D(spacePoints,i,1,cvScalarAll(sP[i].y));
    cvSet2D(spacePoints,i,2,cvScalarAll(sP[i].z));
  }
  cvFindHomography(imagePoints,spacePoints,hImage2Space);
  */
  for (i=0; i<N; i++) {
    printf("%d. (%lf,%lf)->(%lf,%lf,0).\n",i, iP[i].x, 
	   iP[i].y, sP[i].x, sP[i].y);
  }
  cvGetPerspectiveTransform(iP, sP, hImage2Space);

  saveCurrentImage("calibrated");
}

void deleteLastPoint() {
  if (N>0) N--;
  printf("There are %d calibration points so far\n",N);
}

/* do some PTZ based on the action selected*/

#define PANINC 5.0
#define TILTINC 5.0
#define ZOOMINC 10.0

void carryOutPTZ(int action){
  double p,t,z;
  p = myPTZ.getPan(), t = myPTZ.getTilt(), z = myPTZ.getZoom();
  switch (action) {
  case 1: myPTZ.pan(p+PANINC); break;
  case 2: myPTZ.pan(p-PANINC); break;
  case 3: myPTZ.tilt(t+TILTINC); break;
  case 4: myPTZ.tilt(t-TILTINC); break;
  case 5: myPTZ.zoom(z-ZOOMINC); break;
  case 6: myPTZ.zoom(z+ZOOMINC); break;
  case 7: myPTZ.reset(); break;
  case 8: saveCurrentImage("savedimage"); break;
  case 9: printHelp(); break;
  case 10: 
    calFlag=true;
    printCalStart();
    calStart();
    break;
  case 11: 
    deleteLastPoint();
    break;
  case 12: 
    calFlag=false;
    printCalEnd();
    calEnd();
    break;
  case 13: 
    calTest = !calTest;
    if (calTest) {
      printf("Calibration test mode on\n");
      startCalTest();
    }
    break;
  case 14:
    printf("Enter file name to read calibration from?\n");
    scanf("%s",froot);
    readHomography(froot);
  default: break;
  }
  return;
}



//#define CAMNAME "getPictureCam1"
#define CAMNAME "sendVideo"
int main(int argc, char **argv)
{int i;

  Aria::init();
  ArGlobalFunctor1<ArNetPacket *> jpegHandlerCB(&jpegHandler);
  ArNetPacket packet;
  
  std::string hostname;
  int port;

  ArArgumentParser parser(&argc, argv);    // set up our parser
  ArSimpleConnector 
             simpleConnector(&parser);     // set up our simple connector


  printf("Usage: %s [hostname] [port]\n", argv[0]);
  
  if (argc >= 2)
    hostname = argv[1];
  else
    hostname = "localhost";
  
  if (argc >= 3)
    port = atoi(argv[2]);
  else
    port = 7070;


  /* INITIALIZE THE SAVCLIENT AND OPENCV */

  if (!client.blockingConnect(hostname.c_str(), port))
  {
    printf("Could not connect to server, exiting\n");
    exit(1);
  }    

  img = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U, 3);

  cvNamedWindow("Camera",1);

  cvSetMouseCallback("Camera",mouseCallback,(void*)img);

  client.addHandler(CAMNAME, &jpegHandlerCB);

  client.request(CAMNAME, -2); // stop request

  packet.uByteToBuf(50);

  client.runAsync();


  /* INITIALIZATION OF CONNECTION TO ROBOT */

  Aria::init();                             // mandatory init

  parser.loadDefaultArguments();           // load the default arguments 

  if (!simpleConnector.parseArgs()         // check command line args
                || !parser.checkHelpAndWarnUnparsed(1))
  {    
    simpleConnector.logOptions();
    Aria::shutdown();
    return 1;
  }


  if (!simpleConnector.connectRobot(&robot)) // ask for connection to robot
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }

  /* INITIALIZATION OF ROBOT*/

  robot.runAsync(true);
  robot.enableMotors();
  myPTZ.reset(); ArUtil::sleep(1000);
  //Manually set the pan/tilt of camera if desired
  //Otherwise comment out
  myPTZ.panTilt((double)(30), (double)(-16.25));
  printf("%lf pan %lf tilt", myPTZ.getPan(), myPTZ.getTilt());
  ArUtil::sleep(1000);

  int key = 0, action=0;
  do{
   client.requestOnce(CAMNAME, &packet); // request 1 frame with argument of 90

   key = cvWaitKey(33);
   if (key>0) printf("key=%d\n",key);
   action = classifyKeyPress(key);
   if (action>0) carryOutPTZ(action);
   if (readSpaceValues) doReadSpaceValues();
  } while (key != 27 );

  //  client.request(CAMNAME, -1); // request a continuous stream

  client.disconnect();
  client.stopRunning();

  ArUtil::sleep(1000);

  robot.disableMotors();
  robot.disconnect();

  ArUtil::sleep(1000);

  Aria::shutdown();

  return 0;
}



