
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

int writeport(int fd, char *chars);
int readport(int fd, char *result);
int getbaud(int fd);


int fd;

int initport(int fd) {
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 19200...
	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);
	return 1;
}
int writeport(int fd, char *chars) {
	int len = strlen(chars);
	chars[len] = 0x0d; // stick a <CR> after the command
	chars[len+1] = 0x00; // terminate the string properly
	int n = write(fd, chars, strlen(chars));
	if (n < 0) {
		fputs("write failed!\n", stderr);
		return 0;
	}
	return 1;
}

int readport(int fd, char *result) {
	int iIn = read(fd, result, 254);
	result[iIn-1] = 0x00;
	if (iIn < 0) {
		if (errno == EAGAIN) {
			printf("SERIAL EAGAIN ERROR\n");
			return 0;
		} else {
			printf("SERIAL read error %d %s\n", errno, strerror(errno));
			return 0;
		}
	}                    
	return 1;
}

int getbaud(int fd) {
	struct termios termAttr;
	int inputSpeed = -1;
	speed_t baudRate;
	tcgetattr(fd, &termAttr);
	/* Get the input speed.                              */
	baudRate = cfgetispeed(&termAttr);
	switch (baudRate) {
		case B0:      inputSpeed = 0; break;
		case B50:     inputSpeed = 50; break;
		case B110:    inputSpeed = 110; break;
		case B134:    inputSpeed = 134; break;
		case B150:    inputSpeed = 150; break;
		case B200:    inputSpeed = 200; break;
		case B300:    inputSpeed = 300; break;
		case B600:    inputSpeed = 600; break;
		case B1200:   inputSpeed = 1200; break;
		case B1800:   inputSpeed = 1800; break;
		case B2400:   inputSpeed = 2400; break;
		case B4800:   inputSpeed = 4800; break;
		case B9600:   inputSpeed = 9600; break;
		case B115200:  inputSpeed = 115200; break;
		case B19200:  inputSpeed = 19200; break;
		case B38400:  inputSpeed = 38400; break;
	}
	return inputSpeed;
}


#define USBSERIAL "/dev/ttyUSB0"
/*
 NMEA COMMAND $GPGLL,4132.61502,N,07346.15105,W,204042.00,A,D*7C
 */

int main(int argc, char **argv) {
  int count;
	fd = open(USBSERIAL, O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd == -1) {
		perror("open_port: Unable to open port - ");
		return 1;
	} else {
		fcntl(fd, F_SETFL, 0);
	}
	
	initport(fd);
	printf("Set port baud=%d\n", getbaud(fd));
/*
 Example of how to write commands to the GPS 
	char sCmd[254];
	
	sprintf(sCmd,"$JASC,D1,0\r\n"); // turns on diagnostic messages -- don't do this it messes up the interface

	if (!writeport(fd, sCmd)) {
		printf("write failed\n");
		close(fd);
		return 1;
	}

	printf("written:%s\n", sCmd);
	
	*/
	char sResult[254];
	fcntl(fd, F_SETFL, FNDELAY); // don't block serial read


	for (count=0; count<100; count++) {/* read a 100 messages */
	  usleep(500000);/* need to pause a little between them */

	  if (!readport(fd,sResult)) {
		printf("read failed\n");
		close(fd);
		return 1;
	  }
	  /*printf("readport=%s\n", sResult);*/
	  /* example: $GPGLL,4132.61502,N,07346.15105,W,204042.00,A,D*7C */
  
        if (strcmp(sResult,"")!=0) {
          double n,w,d1; char chw, chn,ch1,str1[10];
	         sscanf(sResult,"$GPGLL,%lf,%c,%lf,%c,%lf,%c,%s",
		            &n,&chn,&w,&chw,&d1,&ch1,&str1);
             printf(" north is %lf and west is %lf\n",n,w);
	    }
	}/* end of loop*/

	close(fd);
	return 0;
}

