// inspect.cpp : Defines the entry point for the console application.
//

#include "cv.h"
#include "highgui.h"
#include <stdio.h>

int g_slider_pos =25, g_color_choice=0, g_fname_index=0;
CvScalar g_color_mask;

CvMemStorage *linestorage=0;
CvSeq *lines=0;

#define MAXIMAGES 4
char *flist[MAXIMAGES] = {
	          "outside1.jpg",
                  "outside2.jpg",
                  "outside3.jpg",
		  "outside4.jpg"
				};
IplImage *img, *img1, *imgb1, *imgb2;


void imagePipeline(CvScalar s,int threshold,int findex) {

  if (img!=0) cvReleaseImage(&img);
  img = cvLoadImage(flist[findex]);

  cvZero(img1);
  cvAndS(img,s,img1,0); // mask for the one or more channels
  cvShowImage("image",img1); //and show that

  cvCvtColor(img1,imgb1,CV_BGR2GRAY); // make into gray
  cvThreshold(imgb1,imgb1,threshold,255,CV_THRESH_BINARY); // and then into binary
  cvShowImage("thresholded",imgb1); // and show that

  lines=cvHoughLines2(imgb1,linestorage,CV_HOUGH_PROBABILISTIC, // find all the lines in this binary imake
	  /*pixel res*/5,/*radians res*/CV_PI/180.0,/*threshold*/80,/* min line length, sep*/30,10);
  if (lines!=0) {
	  printf("Number of lines in image = %d\n",lines->total);
	  cvZero(imgb2);
	  for (int i=0; i<lines->total; i++) {
		CvPoint* line = (CvPoint*)cvGetSeqElem(lines,i);
		cvLine( imgb2, line[0], line[1], cvScalarAll(200), 3, 8 );
	  }
  }
  cvShowImage("lines",imgb2); // and finally show the lines
}


void onTrackBarSlide2(int pos){
 g_color_choice=pos;
 if (pos==0) g_color_mask = cvScalar(255,0,0,0);
 else if (pos==1) g_color_mask = cvScalar(0,255,0,0);
 else if (pos==2) g_color_mask = cvScalar(0,0,255,0);
 else g_color_mask = cvScalar(255,255,255,0);

 imagePipeline(g_color_mask, g_slider_pos,g_fname_index);
 return;
}

void onTrackBarSlide1(int pos){
 imagePipeline(g_color_mask, pos, g_fname_index);
 g_slider_pos=pos;

 return;
}

// A Simple Camera Capture Framework
int main(int argc, char* argv[]) {
  int key;

  // Create a window in which the captured images will be presented
  cvNamedWindow( "image", CV_WINDOW_AUTOSIZE );
  cvNamedWindow( "thresholded", CV_WINDOW_AUTOSIZE );
  cvNamedWindow( "lines", CV_WINDOW_AUTOSIZE );
 
  // create trackbars for the windowsdelcaration
  cvCreateTrackbar("thresh","thresholded",&g_slider_pos,255,onTrackBarSlide1);

  cvCreateTrackbar("channel","image",&g_color_choice,4,onTrackBarSlide2);

  //read in the image
  img = cvLoadImage(flist[g_fname_index]);
 
  // make some extra images to be used for further processing
  img1 = cvCreateImage(cvSize(img->width,img->height),IPL_DEPTH_8U,3);
  imgb1 = cvCreateImage(cvSize(img->width,img->height),IPL_DEPTH_8U,1);
  imgb2 = cvCreateImage(cvSize(img->width,img->height),IPL_DEPTH_8U,1);

  //make a CV storage place for the lines from the image
  linestorage = cvCreateMemStorage(0);

  g_color_mask = cvScalar(255,0,0,0); // blue channel
  g_slider_pos=25; //low threshold


  do {
  imagePipeline(g_color_mask, g_slider_pos, g_fname_index);
  key = cvWaitKey(0); // wait and allow user adjustment of sliders & viewing until the ESC key is pressed
  if (key==65363) g_fname_index++;
  if (key==65361) g_fname_index--;
  if (g_fname_index<0) g_fname_index=MAXIMAGES-1; // make a circular list of images
  if (g_fname_index>=MAXIMAGES) g_fname_index=0;
  } while (key!=27);


  cvDestroyWindow( "lines" );
  cvDestroyWindow( "images" );
  cvDestroyWindow( "thresholded" );

  return 0;
}

