#include <iostream>
#include "pcl/io/pcd_io.h"
#include "pcl/point_types.h"
#include "pcl/filters/statistical_outlier_removal.h"

int main (int argc, char** argv)
{

  typedef pcl::PointXYZRGB PointT;

  std::cout << "Statistical Removal Version, first argument input, second argument output; outliers file is written to outliers.pcd" << std::endl;

  int meanK;
  float stdThresh;

  if(argc == 3)
    {
      meanK = 50;
      stdThresh = 1.0;
    }    
  else
    {
      if(argc != 5)
	{
	  std::cout << "Inproper command line arguments. statrm <input.pcd> <output.pcd> <int meanK> <float stdDevThresh>\n";
	  return -1;
	}
      meanK = atol(argv[3]);
      stdThresh = atof(argv[4]);
    }

  std::cout << "Read meanK: " << meanK << "stdDevThresh: " << stdThresh << std::endl;

  pcl::PointCloud<PointT>::Ptr cloud (new pcl::PointCloud<PointT>);
  pcl::PointCloud<PointT>::Ptr cloud_filtered (new pcl::PointCloud<PointT>);



  // Fill in the cloud data
  pcl::PCDReader reader;
  // Replace the path below with the path where you saved your file
  reader.read<PointT> (argv[1], *cloud);

  std::cerr << "Cloud before filtering: " << std::endl;
  std::cerr << *cloud << std::endl;

  // Create the filtering object
  pcl::StatisticalOutlierRemoval<PointT> sor;
  sor.setInputCloud (cloud);

  std::cout << "meanK is:" << meanK;
  sor.setMeanK (meanK);

  std::cout << "std mul thresh is" << stdThresh;
  sor.setStddevMulThresh (stdThresh);
  sor.filter (*cloud_filtered);

  std::cerr << "Cloud after filtering: " << std::endl;
  std::cerr << *cloud_filtered << std::endl;

  pcl::PCDWriter writer;
  writer.write<PointT> (argv[2], *cloud_filtered, true);

  sor.setNegative (true);
  sor.filter (*cloud_filtered);
  writer.write<PointT> ("outliers.pcd", *cloud_filtered, true);

  return (0);
}
