 #include <iostream>
 #include <cstdio>
 #include "pcl/io/pcd_io.h"
 #include "pcl/point_types.h"

#include "pcl/features/normal_3d.h"
#include "pcl/features/fpfh.h"
#include "pcl/registration/registration.h"
#include "pcl/registration/icp.h"
#include "pcl/registration/icp_nl.h"
#include "pcl/registration/ia_ransac.h"

// Add to CmakeLists.txt for pclICP
//
//add_executable (pclICP pclICP.cpp)
//target_link_libraries (pclICP pcl_io pcl_registration)
//


using namespace pcl;
using namespace pcl::io;
using namespace std;

//function to read in a 3x3 matrix to a 3x3 eigen matrix
Eigen::Matrix3f readMat3f(const char* file)
{
  FILE* fd = fopen(file,"r");
  float m1,m2,m3,m4,m5,m6,m7,m8,m9;
  fscanf(fd,"%f,%f,%f,%f,%f,%f,%f,%f,%f",&m1,&m2,&m3,&m4,&m5,&m6,&m7,&m8,&m9);
  fclose(fd);
  Eigen::Matrix3f m(3,3);
  m << m1,m2,m3,m4,m5,m6,m7,m8,m9;
  cout << "Got matrix " << endl << m << endl;
  return m;
}

/*
Eigen::Matrix4f readMat4f(const char* file)
{
  ifstream myfile(file);
  string line;

  getline(myfile,line);

  m << line;
  cout << "got matrix" << m;

  myfile.close();
  return m;
}
*/
int main (int argc, char** argv)
 {
   cout << "This version of ICP takes command line arguments: 1. input pcd filename 2. second input 3. output" << endl;

   sensor_msgs::PointCloud2 cloud_blob1, cloud_blob2, cloud_blob3;
   pcl::PointCloud<pcl::PointXYZ> cloud1,cloud2,cloud3;

   if (pcl::io::loadPCDFile (argv[1], cloud_blob1) == -1)
   {
     std::cerr << "Couldn't read file cloud1.pcd" << std::endl;
     return (-1);
   }
   std::cerr << "Loaded " << cloud_blob1.width * cloud_blob1.height << " data points from cloud1 with the following fields: " << pcl::getFieldsList (cloud_blob1) << std::endl;

   // Convert to the templated message type
   pcl::fromROSMsg (cloud_blob1, cloud1);


   if (pcl::io::loadPCDFile (argv[2], cloud_blob2) == -1)
   {
     std::cerr << "Couldn't read file cloud2.pcd" << std::endl;
     return (-1);
   }
   std::cerr << "Loaded " << cloud_blob2.width * cloud_blob2.height << " data points from cloud2 with the following fields: " << pcl::getFieldsList (cloud_blob2) << std::endl;

    // Convert to the templated message type
   pcl::fromROSMsg (cloud_blob2, cloud2);

  // IterativeClosestPointNonLinear<PointXYZ, PointXYZ> reg;


  // reg.setInputCloud (cloud1.makeShared ());
  // reg.setInputTarget (cloud2.makeShared ());
  // reg.setMaximumIterations (100);
  // reg.setTransformationEpsilon (1e-8);

  // // Register 
  // reg.align (cloud3);
  
  // //EXPECT_EQ ((int)cloud_reg.points.size (), (int)cloud_source.points.size ());

  // Eigen::Matrix4f test = readMat4f("test.txt");

  // Eigen::Matrix3f intrinsic = readMat3f("intrinsic.txt");
  // Eigen::Matrix3f affH = readMat3f("affH.txt");
  // Eigen::Matrix3f kHk = intrinsic.inverse() * affH * intrinsic;
  // cout << endl << "K^-1 H K is: " << endl << kHk << endl << endl;

  // kHk.normalize();
  // cout << "After normalization" << endl << kHk << endl << endl;

  // Eigen::Matrix4f transformation = reg.getFinalTransformation ();
  // cout << transformation << endl;
  // Eigen::Matrix3f rotmat = transformation.block<3,3>(0,0);
  // cout << rotmat << endl;
  // rotmat.normalize();
  // cout << "\n After normalization" << endl;
  // cout << rotmat << endl;

   Eigen::Vector4f newPos(3.0,3.0,3.0,1.0);
   cloud1.sensor_origin_=newPos;

   Eigen::Matrix4f transform;
   transform << 1,0,0,4, 0,1,0,4, 0,0,1,4, 0,0,0,1;
		   
   transformPointCloud(cloud1,cloud3,transform);


  pcl::io::savePCDFileASCII ("cloud1.pcd", cloud1);
  cloud3+=cloud1;
  pcl::io::savePCDFileASCII ("cloud2.pcd", cloud3);

   return (0);
 }
 

