/*frcvlab 2011
 *
 *file history
 *
 * dml spring 2011 - original file
 * sdf summer 2011 - modified to dump to pcd file type directly;
 * corrects error with PointXYZRGBA; we use PointXYZRGB directly and
 * do not have to worry about the bit shift for packing the rgb info
 * sdf 7/26/11 - modified to write binary files to prevent alleged data loss
 *
 * sdf 10/10/11 - seems that gcc 4.6 doesn't like using the point reference
 * trick I was using; updated the includes for API changes from pcl 1.1 to pcl 1.2
 *
 * wrapped conversion into a single function
 * 
 * proposed modifications: 
 *
 * index the points to image coordinates
 * create our own point type that maintains disparity
 * 
 */

//#include <iostream>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>

#include <stdio.h>

namespace frcv
{
  void rcv2pcd(const char* src, pcl::PointCloud<pcl::PointXYZRGB>::Ptr& dst)
  {
    int numPoints = 0;
    struct mypoint {
      double x,y,z;
      int r,g,b;
      int u,v,d;
    } p;
  
    FILE* in = fopen(src,"r");
    while (fscanf(in,"%lf, %lf, %lf, %d, %d, %d, %d, %d, %d\n",
		  &p.x, &p.y, &p.z, &p.u, &p.v, &p.d, &p.r, &p.g, &p.b)!=EOF /*&& k<320*240*/) {
      numPoints++;
    }
    fclose(in);
    printf("%d points to be converted\n",numPoints);

    //create a temporary cloud and set the details
    pcl::PointCloud<pcl::PointXYZRGB>::Ptr cloud(new pcl::PointCloud<pcl::PointXYZRGB>);
    cloud->width = numPoints;
    cloud->height = 1;
    cloud->points.resize(numPoints);

    //read in the points
    in=fopen(src,"r");
    int i=0;
    while (fscanf(in,"%lf, %lf, %lf, %d, %d, %d, %d, %d, %d\n",
		  &p.x, &p.y, &p.z, &p.u, &p.v, &p.d, &p.r, &p.g, &p.b)!=EOF /*&& k<320*240*/) {

      pcl::PointXYZRGB& pt = cloud->points[i];
      pt.x = p.x;
      pt.y = p.y;
      pt.z = p.z;
      pt.r = p.r;
      pt.g = p.g;
      pt.b = p.b;
      i++;
    }
    fclose(in);
  
    dst.swap(cloud);
    printf("Conversion done\n");
  }
}

int main(int argc, char **argv) {
	if (argc!=3) {
		 printf("command line arguments infile, outfile missing\n");
		 return(0);
	}

    printf("Translating point cloud format in %s to %s.\n",argv[1],argv[2]);

    //destination cloud
    pcl::PointCloud<pcl::PointXYZRGB>::Ptr cloud;
    //convert to pcl point cloud
    frcv::rcv2pcd(argv[1],cloud);

    /*do some cloud processing here*/

    //write as ascii pcd file
    pcl::io::savePCDFile(argv[2], /*filename*/
			 *cloud, /*cloud*/
			 true); /*false = ascii; true = binary; note the ascii version truncates the floating point values which causes "significant" color loss, as I have read on the mailing list. sdf 7/26/11*/

     printf("conversion done\n");

}
