#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include <string.h>

#include "Aria.h"
#include "ArNetworking.h"

#include <cv.h>
#include <highgui.h>

/** @example getVideoExample.cpp
 *
 *  This simple example requests video images from the server
 *  and saves them repeatedly to a file named "camera.jpg".
 *  Connect to SAVserver, ACTS, or a custom video server.
 *  It uses OpenCv to open a window and display the latest camera image
 */

ArClientBase client;

int global_count=0; /* index for frames */
int global_max = 20; /* max number of frames to store */
char froot[100] = "frame"; /* file name root to use */
char fname[100]; 

IplImage *img;

unsigned char jpeg[50000];
int jpegSize;

ArRobot robot;                           // declare robot object

//ArDPPTU myPTZ(&robot);
ArVCC4 myPTZ(&robot);

/*
 Write JPEG data to a file
 */
void saveJpeg(char *fname, unsigned char *jdata, int jsize) {
  FILE *file;

  if ((file = fopen(fname, "wb+")) != NULL)
  {
    fwrite(jdata, jsize, 1, file);
    fclose(file);
  }
  else
    printf("Could not write jpeg file %s\n",fname);
}

/*
 Slow version: stores jpeg in file and then loads into open cv
 */

void jpegHandler(ArNetPacket *packet)
{
  unsigned int width;
  unsigned int height;

  width = packet->bufToUByte2();
  height = packet->bufToUByte2();
  //printf("Got jpeg of %d x %d\n", width, height);
  jpegSize = packet->getDataLength() - packet->getDataReadLength();
  //printf("Jpeg %d long\n", jpegSize);
  packet->bufToData((char *)jpeg, jpegSize);
  sprintf(fname,"camera.jpg",froot);
  saveJpeg(fname,jpeg,jpegSize);
  img = cvLoadImage(fname);
  cvShowImage("Camera",img);
}

void saveCurrentImage(char *savedname) {

  sprintf(froot,"%s_%d.jpg",savedname,global_count++);
  saveJpeg(froot,jpeg,jpegSize);
  printf("Current image saved to %s\n",froot,'\n');

}

/*
 classify key into the arrow keys and +/- keys and a few others
 */

int classifyKeyPress(int key) {
  int ret=-1;
    if (key>0) {
      if (key==65363 || key==65430) ret = 1; //printf("left arrow\n");
      if (key==65361 || key==65432) ret = 2; //printf("right arrow\n");
      if (key==65362 || key==65431) ret = 3; //printf("up arrow\n");
      if (key==65364 || key==65433) ret = 4; //printf("left arrow\n");
      if (key==65631 || key==65453) ret = 5; //printf("minus\n");
      if (key==65579 || key==65451) ret = 6; //printf("plus\n");
      if (key==65360 )              ret = 7; //printf("home\n");
      if (key==   10 )              ret = 8; //printf("ret\n");
      if (key==65599 )              ret = 9; //printf("question mark\n");
    }
    return ret;
}


void printHelp() {
  printf("Select the video window and press one of the following:\n");
  printf(" left arrow pans left \n");
  printf(" right arrow pans right \n");
  printf(" up arrow tilts up \n");
  printf(" down arrow tilts down \n");
  printf(" + zooms in \n");
  printf(" - zooms out \n");
  printf(" home resets the camera (takes a minute or so) \n");
  printf(" return saves an image to savedimage<n>.jpg where n increments \n");
  printf(" ? prints this message \n");
}

/* do some PTZ based on the action selected*/

#define PANINC 5.0
#define TILTINC 5.0
#define ZOOMINC 10.0

void carryOutPTZ(int action){
  double p,t,z;
  p = myPTZ.getPan(), t = myPTZ.getTilt(), z = myPTZ.getZoom();
  switch (action) {
  case 1: myPTZ.pan(p+PANINC); break;
  case 2: myPTZ.pan(p-PANINC); break;
  case 3: myPTZ.tilt(t+TILTINC); break;
  case 4: myPTZ.tilt(t-TILTINC); break;
  case 5: myPTZ.zoom(z-ZOOMINC); break;
  case 6: myPTZ.zoom(z+ZOOMINC); break;
  case 7: myPTZ.reset(); break;
  case 8: saveCurrentImage("savedimage"); break;
  case 9: printHelp(); break;
  default: break;
  }
  return;
}



//#define CAMNAME "getPictureCam1"
#define CAMNAME "sendVideo"
int main(int argc, char **argv)
{int i;

  Aria::init();
  ArGlobalFunctor1<ArNetPacket *> jpegHandlerCB(&jpegHandler);
  ArNetPacket packet;
  
  std::string hostname;
  int port;

  ArArgumentParser parser(&argc, argv);    // set up our parser
  ArSimpleConnector 
             simpleConnector(&parser);     // set up our simple connector


  printf("Usage: %s [hostname] [port]\n", argv[0]);
  
  if (argc >= 2)
    hostname = argv[1];
  else
    hostname = "localhost";
  
  if (argc >= 3)
    port = atoi(argv[2]);
  else
    port = 7070;


  /* INITIALIZE THE SAVCLIENT AND OPENCV */

  if (!client.blockingConnect(hostname.c_str(), port))
  {
    printf("Could not connect to server, exiting\n");
    exit(1);
  }    

  img = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U, 3);

  cvNamedWindow("Camera",1);

  client.addHandler(CAMNAME, &jpegHandlerCB);

  client.request(CAMNAME, -2); // stop request

  packet.uByteToBuf(50);

  client.runAsync();


  /* INITIALIZATION OF CONNECTION TO ROBOT */

  Aria::init();                             // mandatory init

  parser.loadDefaultArguments();           // load the default arguments 

  if (!simpleConnector.parseArgs()         // check command line args
                || !parser.checkHelpAndWarnUnparsed(1))
  {    
    simpleConnector.logOptions();
    Aria::shutdown();
    return 1;
  }


  if (!simpleConnector.connectRobot(&robot)) // ask for connection to robot
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }

  /* INITIALIZATION OF ROBOT*/

  robot.runAsync(true);
  robot.enableMotors();
  myPTZ.reset(); ArUtil::sleep(5000);
      myPTZ.panTilt((double)(0), (double)(-7.25));
	ArUtil::sleep(5000);


  int key = 0, action=0;
  do{
   client.requestOnce(CAMNAME, &packet); // request 1 frame with argument of 90

   key = cvWaitKey(33);
   action = classifyKeyPress(key);
   if (action>0) carryOutPTZ(action);

  } while (key != 27 );

  //  client.request(CAMNAME, -1); // request a continuous stream

  client.disconnect();
  client.stopRunning();

  Aria::shutdown();
  return 0;
}



