/*
MobileRobots Advanced Robotics Interface for Applications (ARIA)
Copyright (C) 2004, 2005 ActivMedia Robotics LLC
Copyright (C) 2006, 2007 MobileRobots Inc.

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
MobileRobots for information about a commercial version of ARIA at 
robots@mobilerobots.com or 
MobileRobots Inc, 19 Columbia Drive, Amherst, NH 03031; 800-639-9481

Some small modifications for convenience by DamianM. Lyons 2009 Fordham RCV

*/

#include "Aria.h"
#include "ArNetworking.h"

/** @example getVideoExample.cpp
 *
 *  This simple example requests video images from the server
 *  and saves them repeatedly to a file named "<filename>_<framenum>.jpg".
 *  Connect to SAVserver, ACTS, or a custom video server.
 * it requests the name and the number of frames up front from the user
 */
ArClientBase client;

int global_count=0; /* index for frames */
int global_max = 20; /* max number of frames to store */
char froot[100] = "frame"; /* file name root to use */
char fname[100]; 

void jpegHandler(ArNetPacket *packet)
{
  unsigned int width;
  unsigned int height;
  unsigned char jpeg[50000];
  int jpegSize;
  FILE *file;

  width = packet->bufToUByte2();
  height = packet->bufToUByte2();
  printf("Got jpeg of %d x %d\n", width, height);
  jpegSize = packet->getDataLength() - packet->getDataReadLength();
  printf("Jpeg %d long\n", jpegSize);
  packet->bufToData((char *)jpeg, jpegSize);

  sprintf(fname,"%s_%d.jpg",froot,global_count++);
  if ((file = fopen(fname, "wb+")) != NULL)
  {
    fwrite(jpeg, jpegSize, 1, file);
    fclose(file);
  }
  else
    printf("Could not write file\n");
  
  if (global_count==global_max) {
  client.disconnect();
  client.stopRunning();
  }
}

//#define CAMNAME "getPictureCam1"
#define CAMNAME "sendVideo"
int main(int argc, char **argv)
{int i;

  Aria::init();
  ArGlobalFunctor1<ArNetPacket *> jpegHandlerCB(&jpegHandler);
  ArNetPacket packet;
  
  std::string hostname;
  int port;

  printf("Usage: %s [hostname] [port]\n", argv[0]);
  
  if (argc >= 2)
    hostname = argv[1];
  else
    hostname = "localhost";
  
  if (argc >= 3)
    port = atoi(argv[2]);
  else
    port = 7070;

  if (!client.blockingConnect(hostname.c_str(), port))
  {
    printf("Could not connect to server, exiting\n");
    exit(1);
  }    


  printf("Enter to root file name to use for all frames: \n");
  scanf("%s",froot);
  printf("Enter the number of frames to capture: \n");
  scanf("%d",&global_max);
  if (global_max<=0 && global_max>1000) return 0;

  printf("OK: frames will be %s_0 to %s_%d\n",froot,froot,global_max-1);

  client.addHandler(CAMNAME, &jpegHandlerCB);

  client.request(CAMNAME, -2); // stop request

  packet.uByteToBuf(50);

  for (i=0; i<global_max; i++)
   client.requestOnce(CAMNAME, &packet); // request 1 frame with argument of 90

  //  client.request(CAMNAME, -1); // request a continuous stream

  client.run();


  Aria::shutdown();
  return 0;
}



